/*****************************************************************************
 * leco                                                                      *
 *****************************************************************************/

/*****************************************************************************
 * File name:     src/block.c                                                *
 * Created:       2006-07-03 by Hampa Hug <hampa@hampa.ch>                   *
 * Copyright:     (C) 2006-2007 Hampa Hug <hampa@hampa.ch>                   *
 *****************************************************************************/

/*****************************************************************************
 * This program is free software. You can redistribute it and / or modify it *
 * under the terms of the GNU General Public License version 2 as  published *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope  that  it  will  be  useful,  but *
 * WITHOUT  ANY   WARRANTY,   without   even   the   implied   warranty   of *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  General *
 * Public License for more details.                                          *
 *****************************************************************************/

/* $Id: block.c 77 2007-08-05 14:49:12Z hampa $ */


#include "config.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "lstdint.h"

#include "gf.h"
#include "block.h"


gf_t par_gf;


#if defined(LECO_ARCH_PPC) || defined(LECO_ARCH_SPARC)

static inline
uint16_t lec_get_uint16_be (const void *buf, unsigned idx)
{
	const uint16_t *src;

	src = (const uint16_t *) ((const unsigned char *) buf + idx);

	return (*src);
}

#else

static inline
uint16_t lec_get_uint16_be (const void *buf, unsigned idx)
{
	const unsigned char *src;

	src = (const unsigned char *) buf + idx;

	return ((src[0] << 8) | src[1]);
}

#endif

#if defined(LECO_ARCH_PPC) || defined(LECO_ARCH_SPARC)

static inline
void lec_set_uint16_be (void *buf, unsigned idx, uint16_t val)
{
	uint16_t *dst;

	dst = (uint16_t *) ((unsigned char *) buf + idx);

	*dst = val;
}

#else

static inline
void lec_set_uint16_be (void *buf, unsigned idx, uint16_t val)
{
	unsigned char *dst;

	dst = (unsigned char *) buf + idx;

	dst[0] = (val >> 8) & 0xff;
	dst[1] = val & 0xff;
}

#endif

void lec_block_clear (uint16_t *blk, unsigned long bsize)
{
	unsigned i;

	for (i = 0; i < bsize; i++) {
		blk[i] = 0;
	}
}

void lec_block_get (uint16_t *dst, const void *src, unsigned long bsize)
{
	unsigned i;

	for (i = 0; i < bsize; i++) {
		dst[i] = lec_get_uint16_be (src, 2 * i);
	}
}

void lec_block_set (void *dst, const uint16_t *src, unsigned long bsize)
{
	unsigned i;

	for (i = 0; i < bsize; i++) {
		lec_set_uint16_be (dst, 2 * i, src[i]);
	}
}

void lec_block_mul (uint16_t *blk, unsigned short x, unsigned long bsize)
{
	unsigned i;

	for (i = 0; i < bsize; i++) {
		blk[i] = gf_mul (&par_gf, blk[i], x);
	}
}

void lec_block_div (uint16_t *blk, unsigned short x, unsigned long bsize)
{
	unsigned i;

	for (i = 0; i < bsize; i++) {
		blk[i] = gf_div (&par_gf, blk[i], x);
	}
}

void lec_block_add (uint16_t *dst, const uint16_t *src, unsigned long bsize)
{
	unsigned i;

	for (i = 0; i < bsize; i++) {
		dst[i] ^= src[i];
	}
}

void lec_block_sub (uint16_t *dst, const uint16_t *src, unsigned long bsize)
{
	unsigned i;

	for (i = 0; i < bsize; i++) {
		dst[i] ^= src[i];
	}
}

void lec_block_muladd (uint16_t *dst, const uint16_t *src, uint16_t x, unsigned long bsize)
{
	unsigned      i;
	unsigned long j;
	unsigned      tx;
	unsigned long ty;

	if (x == 0) {
		return;
	}

	ty = par_gf.log[x];

	for (i = 0; i < bsize; i++) {
		tx = src[i];

		if (tx == 0) {
			continue;
		}

		tx = par_gf.log[tx];

		j = tx + ty;

		if (j > 65534) {
			j -= 65535;
		}

		dst[i] ^= par_gf.exp[j];
	}
}

uint16_t lec_get_coeff (unsigned row, unsigned col)
{
	unsigned long x;

	x = (unsigned long) row * (unsigned long) col;

	return (par_gf.exp[x % par_gf.mask]);
}
