/*****************************************************************************
 * leco                                                                      *
 *****************************************************************************/

/*****************************************************************************
 * File name:     src/indent/src/gf.new.c                                    *
 * Created:       2005-08-07 by Hampa Hug <hampa@hampa.ch>                   *
 * Last modified: 2005-08-09 by Hampa Hug <hampa@hampa.ch>                   *
 * Copyright:     (C) 2005 Hampa Hug <hampa@hampa.ch>                        *
 *****************************************************************************/

/*****************************************************************************
 * This program is free software. You can redistribute it and / or modify it *
 * under the terms of the GNU General Public License version 2 as  published *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope  that  it  will  be  useful,  but *
 * WITHOUT  ANY   WARRANTY,   without   even   the   implied   warranty   of *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  General *
 * Public License for more details.                                          *
 *****************************************************************************/

/* $Id: gf.c 40 2005-12-28 21:36:26Z hampa $ */


#include <stdlib.h>

#include "gf.h"


static unsigned long gf_poly[17] = {
	0,         0,         GF2_POLY,  GF3_POLY,
	GF4_POLY,  GF5_POLY,  GF6_POLY,  GF7_POLY,
	GF8_POLY,  GF9_POLY,  GF10_POLY, GF11_POLY,
	GF12_POLY, GF13_POLY, GF14_POLY, GF15_POLY,
	GF16_POLY
};


int gf_init (gf_t *gf, unsigned k)
{
	unsigned long i, n;

	if ((k > 16) || (gf_poly[k] == 0)) {
		return (1);
	}

	gf->k = k;
	gf->poly = gf_poly[k];
	gf->high = 1U << (k - 1);
	gf->mask = gf->high | (gf->high - 1);

	n = 1UL << k;

	gf->exp = malloc (2 * n * sizeof (unsigned short));
	if (gf->exp == NULL) {
		return (1);
	}

	gf->log = gf->exp + n;

	gf->exp[0] = 1;

	for (i = 1; i < n; i++) {
		if (gf->exp[i - 1] & gf->high) {
			gf->exp[i] = (gf->exp[i - 1] << 1) ^ gf->poly;
		}
		else {
			gf->exp[i] = (gf->exp[i - 1] << 1);
		}
	}

	gf->exp[n - 1] = 0;

	for (i = 0; i < n; i++) {
		gf->log[gf->exp[i]] = i;
	}

	return (0);
}

gf_t *gf_new (unsigned k)
{
	gf_t *gf;

	gf = malloc (sizeof (gf_t));
	if (gf == NULL) {
		return (NULL);
	}

	if (gf_init (gf, k)) {
		free (gf);
		return (NULL);
	}

	return (gf);
}

void gf_free (gf_t *gf)
{
	if (gf != NULL) {
		free (gf->exp);
	}
}

void gf_del (gf_t *gf)
{
	if (gf != NULL) {
		gf_free (gf);
		free (gf);
	}
}

unsigned short gf_add (const gf_t *gf, unsigned short x, unsigned short y)
{
	return ((x ^ y) & gf->mask);
}

unsigned short gf_sub (const gf_t *gf, unsigned short x, unsigned short y)
{
	return ((x ^ y) & gf->mask);
}

unsigned short gf_inv (const gf_t *gf, unsigned short x)
{
	x &= gf->mask;

	if (x == 0) {
		return (gf->mask);
	}

	if (x == 1) {
		return (1);
	}

	return (gf->exp[gf->mask - gf->log[x]]);
}

#if 0
unsigned short gf_mul (const gf_t *gf, unsigned short x, unsigned short y)
{
	unsigned i;

	x &= gf->mask;
	y &= gf->mask;

	if ((x == 0) || (y == 0)) {
		return (0);
	}

	x = gf->log[x];
	y = gf->log[y];

	/* i = (x + y) % gf->mask; */

	if ((gf->mask - y) <= x) {
		i = x + y - gf->mask;
	}
	else {
		i = x + y;
	}

	return (gf->exp[i]);
}
#endif

unsigned short gf_div (const gf_t *gf, unsigned short x, unsigned short y)
{
	unsigned i;

	x &= gf->mask;
	y &= gf->mask;

	if (x == 0) {
		return (0);
	}

	if (y == 0) {
		return (gf->mask);
	}

	x = gf->log[x];
	y = gf->log[y];

	if (x < y) {
		i = gf->mask + x - y;
	}
	else {
		i = x - y;
	}

	return (gf->exp[i]);
}
