#!/bin/bash

# $Id: lecos.sh 70 2007-02-06 12:30:52Z hampa $


case "$1" in
"--version")
	echo "lecos version %LECO_VERSION_STR (%LECO_CFG_DATE %LECO_CFG_TIME)"
	echo ""
	echo "Copyright (C) 2005-2007 by Hampa Hug <hampa@hampa.ch>"
	exit 0
	;;
"--help")
	echo "usage: lecos [options] [files...]"
	echo "  -f, --first int        Set first ecc block [0]"
	echo "  -m, --memory int       Set memory size [64M]"
	echo "  -n, --data-blocks int  Set number of data blocks [2048]"
	echo "  -o, --output string    Set output file name"
	echo "  -p, --percentage int   Set ecc percentage of data size"
	echo "  -s, --ecc-size int     Set ecc size [8M]"
	echo "  -t, --total-size int   Set ecc + data size"
	echo "  -v, --verbose          Verbose operation [no]"
	echo "  -y, --force            Overwrite existing files [no]"
	exit 0
	;;
esac

par_verbose=0
par_force=0
par_bcnt=2048
par_size=8M
par_mem=64M
par_first=0
par_pct=""
par_up=""
par_out=""

while test $# -gt 0 ; do
	src=$1
	shift 1

	case "$src" in
	"-f" | "--first")
		par_first=$1
		shift 1
		continue
		;;
	"-m" | "--memory")
		par_mem=$1
		shift 1
		continue
		;;
	"-n" | "--data-blocks")
		par_bcnt=$1
		shift 1
		continue
		;;
	"-o" | "--output")
		par_out=$1
		shift 1
		continue
		;;
	"-p" | "--percentage")
		par_pct=$1
		par_size=""
		par_up=""
		shift 1
		continue
		;;
	"-s" | "--ecc-size")
		par_size=$1
		par_up=""
		par_pct=""
		shift 1
		continue
		;;
	"-t" | "--total-size")
		par_up=$1
		par_size=""
		par_pct=""
		shift 1
		continue
		;;
	"-v" | "--verbose")
		par_verbose=1
		continue
		;;
	"-y" | "--force")
		par_force=1
		continue
		;;
	esac

	# check if source exists
	if test -d "$src" -o ! -r "$src" ; then
		echo "$0: skipping $src" >&2
		continue
	fi

	# determine destination directory and file name
	if test -n "$par_out" ; then
		dst="$par_out"
	elif test -d "ecc" ; then
		dst="ecc/${src%.*}.erc"
	else
		dst="${src%.*}.erc"
	fi

	# check if destination file exists
	if test -e "$dst" ; then
		if test "x$par_force" != "x1" ; then
			echo "$0: $dst exists, skipping $src -> $dst" >&2
			continue
		fi
	fi

	# get ecc size
	if test -n "$par_pct" ; then
		size=$(stat -c "%s" "$src")
		size=$((($size*$par_pct)/100))
		size="-s $size"
	elif test -n "$par_up" ; then
		size="-t $par_up"
	else
		size="-s $par_size"
	fi

	echo "$src -> $dst"

	if test "x$par_verbose" != "x0" ; then
		echo "leco -e -r -f $par_first -B $par_mem -b 16M -n $par_bcnt $size -i \"$src\" -l \"$dst\""
	fi

	leco -e -r -f $par_first -B $par_mem -b 16M -n $par_bcnt $size -i "$src" -l "$dst"

	chmod --reference="$src" "$dst" > /dev/null 2>&1
	touch --reference="$src" "$dst" > /dev/null 2>&1
done
