/*****************************************************************************
 * leco                                                                      *
 *****************************************************************************/

/*****************************************************************************
 * File name:     src/mt19937.c                                              *
 * Created:       2006-08-16 by Hampa Hug <hampa@hampa.ch>                   *
 * Copyright:     (C) 2006-2007 Hampa Hug <hampa@hampa.ch>                   *
 *****************************************************************************/

/*****************************************************************************
 * This program is free software. You can redistribute it and / or modify it *
 * under the terms of the GNU General Public License version 2 as  published *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope  that  it  will  be  useful,  but *
 * WITHOUT  ANY   WARRANTY,   without   even   the   implied   warranty   of *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  General *
 * Public License for more details.                                          *
 *****************************************************************************/

/* $Id: mt19937.c 79 2007-08-05 15:05:05Z hampa $ */


#include "mt19937.h"


#define MT19937_M        397
#define MT19937_K        (MT19937_N - MT19937_M)
#define MT19937_MATRIX_A 0x9908b0dfUL


void mt_init_uint32 (mt19937_t *mt, unsigned long val)
{
	unsigned      i;
	unsigned long *st;

	st = &mt->state[0];

	st[0] = val & 0xffffffff;

	for (i = 1; i < MT19937_N; i++) {
		st[i] = 1812433253UL * (st[i - 1] ^ (st[i - 1] >> 30)) + i;
		st[i] &= 0xffffffff;
	}

	mt->index = MT19937_N;
}

static
unsigned long mt_get_seed (const void *seed, unsigned cnt, unsigned idx)
{
	unsigned            i;
	unsigned long       v;
	const unsigned char *tmp;

	tmp = (const unsigned char *) seed;

	idx = idx << 2;

	v = 0;
	for (i = 0; i < 4; i++) {
		v = v << 8;

		if (idx < cnt) {
			v |= (tmp[idx++] & 0xff);
		}
	}

	return (v);
}

void mt_init (mt19937_t *mt, const void *seed, unsigned cnt)
{
	unsigned      i, j, n;
	unsigned      cnt4;
	unsigned long *st;

	st = &mt->state[0];

	mt_init_uint32 (mt, 19650218);

	cnt4 = (cnt + 3) >> 2;

	i = 1;
	j = 0;
	n = (cnt4 < MT19937_N) ? MT19937_N : cnt4;

	while (n > 0) {
		st[i] ^= 1664525UL * (st[i - 1] ^ (st[i - 1] >> 30));
		st[i] += mt_get_seed (seed, cnt, j) + j;
		st[i] &= 0xffffffff;

		i += 1;
		j += 1;

		if (i >= MT19937_N) {
			st[0] = st[MT19937_N - 1];
			i = 1;
		}

		if (j >= cnt4) {
			j = 0;
		}

		n -= 1;
	}

	for (n = 0; n < (MT19937_N - 1); n++) {
		st[i] ^= 1566083941UL * (st[i - 1] ^ (st[i - 1] >> 30));
		st[i] -= i;
		st[i] &= 0xffffffff;

		i += 1;
		if (i >= MT19937_N) {
			st[0] = st[MT19937_N - 1];
			i = 1;
		}
	}

	st[0] = 0x80000000;
}

void mt_copy (mt19937_t *dst, const mt19937_t *src)
{
	*dst = *src;
}

static
void mt_calc (mt19937_t *mt)
{
	unsigned      i;
	unsigned long v;
	unsigned long *st;

	st = &mt->state[0];

	for (i = 0; i < MT19937_K; i++) {
		v = (st[i] & 0x80000000) | (st[i + 1] & 0x7fffffff);
		st[i] = st[i + MT19937_M] ^ (v >> 1);
		if (v & 1) {
			st[i] ^= MT19937_MATRIX_A;
		}
	}

	for (i = MT19937_K; i < (MT19937_N - 1); i++) {
		v = (st[i] & 0x80000000) | (st[i + 1] & 0x7fffffff);
		st[i] = st[i - MT19937_K] ^ (v >> 1);
		if (v & 1) {
			st[i] ^= MT19937_MATRIX_A;
		}
	}

	v = (st[MT19937_N - 1] & 0x80000000) | (st[0] & 0x7fffffff);
	st[MT19937_N - 1] = st[MT19937_M - 1] ^ (v >> 1);
	if (v & 1) {
		st[MT19937_N - 1] ^= MT19937_MATRIX_A;
	}

	mt->index = 0;
}

unsigned long mt_get_uint32 (mt19937_t *mt)
{
	unsigned long v;

	if (mt->index >= MT19937_N) {
		mt_calc (mt);
	}

	v = mt->state[mt->index++];

	v ^= (v >> 11);
	v ^= (v << 7) & 0x9d2c5680;
	v ^= (v << 15) & 0xefc60000;
	v ^= (v >> 18);

	return (v);
}
